#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

#include <Cg/cg.h>
#include <Cg/cgGL.h>

#include <stdlib.h>
#include <stdio.h>

CGcontext	cgContext;
CGprogram	cgProgram;
CGprofile	cgVertexProfile;
CGparameter	modelViewMatrix, position, color;

void init(){

  glEnable(GL_DEPTH_TEST);
	
	cgContext = cgCreateContext();	
	if(cgContext==0){
	  printf("Nie można utworzyć kontekstu Cg!");
	  exit(1);
	}
	
	cgVertexProfile = cgGLGetLatestProfile(CG_GL_VERTEX);
	if(cgVertexProfile == CG_PROFILE_UNKNOWN){
	  printf("Nieznany profil!");
	  exit(1);
	}
	
	cgGLSetOptimalOptions(cgVertexProfile);	
	
	cgProgram = cgCreateProgramFromFile(cgContext, CG_SOURCE, "shader.cg", cgVertexProfile, "main", 0);
	
	if (cgProgram == 0)
	{
		// We Need To Determine What Went Wrong
		CGerror Error = cgGetError();

		// Show A Message Box Explaining What Went Wrong
		fprintf(stderr,"%s \n",cgGetErrorString(Error));
		exit(-1);													// We Cannot Continue
	}
	
	cgGLLoadProgram(cgProgram);
	position		= cgGetNamedParameter(cgProgram, "IN.position");
	color			= cgGetNamedParameter(cgProgram, "IN.color");
	modelViewMatrix		= cgGetNamedParameter(cgProgram, "ModelViewProj");
}

void display() {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glLoadIdentity();

  gluLookAt(50.0f, 250.0f, -845.0f, 0.0f, 0.0f, 0.0f, 0, 1, 0);  
  cgGLSetStateMatrixParameter(modelViewMatrix, CG_GL_MODELVIEW_PROJECTION_MATRIX, CG_GL_MATRIX_IDENTITY);	
  cgGLEnableProfile(cgVertexProfile);	
  cgGLBindProgram(cgProgram);
  cgGLSetParameter4f(color, 0.5f, 1.0f, 0.5f, 1.0f);
 
  glColor3f(1, 1, 0);
  glutSolidCube(250);  
  cgGLDisableProfile(cgVertexProfile);	
  
  glutSwapBuffers();
}


void reshape(int w, int h) {
  glViewport(0, 0, (GLsizei) w, (GLsizei) h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective (45.0f, (GLfloat)(w)/(GLfloat)(h), 0.1f, 5000.0f);  
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
}
// funkcja obsługi klawiatury
void klawiatura(unsigned char key, int x, int y)
{
  switch (key)
  {
    case 27 :
    case 'q':
      cgDestroyContext(cgContext);
      exit(0);
    break;
  }
}

void funkcjaTimera(int val) {
	// odrysowanie ekranu po wprowadzeniu zmian (jeśli jest taka potrzeba):
	glutPostRedisplay();
	// ponowne ustawienie timera (co 1000 ms = 1s):
	glutTimerFunc(100, funkcjaTimera, 0);
}

int main(int argc, char *argv[]) {
  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH); // ustawienie podwójnego  buforowania i kolorów RGB
  glutInitWindowSize(640, 480);
  // rozmiar okna
  glutCreateWindow(argv[0]);
  glutDisplayFunc(display);
  // ustawienie funkcji odpowiedzialnej za  rysowanie
  glutReshapeFunc(reshape);
  // ustawienie funkcji wywoływanej przy zmianie rozmiaru okna
  glutKeyboardFunc(klawiatura);
  // ustawienie funkcji obsługi klawiatury
  init();
  glutTimerFunc(100, funkcjaTimera, 0);
  glutMainLoop();
  // wejście do głównej pętli programu
  return 0;
}
 

